from subprocess import Popen, PIPE
from testtools.content import text_content
from testtools.matchers import Contains
from functools import wraps
import platform
import errno
import os
from time import sleep, time
import signal


CLUSTER_CONTROLLER_IP = '192.168.20.5'
SQUID_DEB_PROXY_URL = 'http://10.98.3.6:8000'
# HTTP proxy used to access the external world.
HTTP_PROXY = "http://10.98.3.6:3128"


class TimeoutError(Exception):
    pass


def is_juju_core():
    """Report if 'juju' is provider by juju-core."""
    retcode, out, err = run_command(['juju', '--version'])
    # pyjuju 0.6 has no --version option.  pyjuju 0.7 returns "juju
    # <version>".  gojuju returns "<version>".
    if out == '' or 'juju ' in out:
        return False
    return True


def is_dist(distname):
    _, _, distid = platform.linux_distribution()
    return distid == distname


def is_precise():
    return is_dist('precise')


def is_saucy():
    return is_dist('saucy')

# Timeout decorator from
# http://stackoverflow.com/questions/2281850/timeout-function-if-it-takes-too-long-to-finish
def timeout(seconds=10, error_message=os.strerror(errno.ETIME)):
    def decorator(func):
        def _handle_timeout(signum, frame):
            raise TimeoutError(error_message)

        def wrapper(*args, **kwargs):
            signal.signal(signal.SIGALRM, _handle_timeout)
            signal.alarm(seconds)
            try:
                result = func(*args, **kwargs)
            finally:
                signal.alarm(0)
            return result

        return wraps(func)(wrapper)

    return decorator


def run_command(args, env=None):
    """A wrapper to Popen to run commands in the command-line."""
    process = Popen(args, stdout=PIPE, stderr=PIPE, stdin=PIPE, env=env)
    stdout, stderr = process.communicate()
    return process.returncode, stdout, stderr


def update_pxe_config():
    content = open('/etc/maas/import_pxe_files').read()
    content = content.replace(
        '#RELEASES="precise quantal raring saucy"',
        'RELEASES="precise saucy"')
    with open('/etc/maas/import_pxe_files', 'w') as f:
        f.write(content)


def retries(timeout=30, delay=1):
    """Helper for retrying something, sleeping between attempts.

    Yields ``(elapsed, remaining)`` tuples, giving times in seconds.

    @param timeout: From now, how long to keep iterating, in seconds.
    @param delay: The sleep between each iteration, in seconds.
    """
    start = time()
    end = start + timeout
    for now in iter(time, None):
        if now < end:
            yield now - start, end - now
            sleep(min(delay, end - now))
        else:
            break


def assertStartedUpstartService(runner, service_name, log_file=None):
    """Assert that the upstart service 'service_name' is running."""
    if log_file is not None:
        try:
            log_file_content = file(log_file).read()
            runner.addDetail(
                "File %s content" % log_file,
                text_content(log_file_content))
        except IOError:
            runner.addDetail(
                "File %s content" % log_file,
                text_content("could not be read"))
    # The service might be running pre-start, retry a few times
    # before giving up.
    expected = service_name + " start/running"
    for elapsed, remaining in retries(timeout=30, delay=5):
        retcode, output, err = run_command(["service", service_name, "status"])
        if expected in output:
            break

    runner.assertThat(output, Contains(expected))


def assertCommandReturnCode(
    runner, cmd, expected_output, env=None, expected_retcode=0):
    """Assert that cmd returns appropriate return code."""
    if env is None:
        env = os.environ.copy()
    retcode, output, err = run_command(cmd, env=env)
    runner.addDetail('%s stdout' % cmd, text_content(str(output)))
    runner.addDetail('%s stderr' % cmd, text_content(str(err)))
    runner.assertThat(output, Contains(expected_output))
    runner.assertIs(retcode, expected_retcode)
